@@ -6,13 +6,13 @@ import traceback
 import types
 import warnings
 
+import eventlet
+from eventlet import greenio
+from eventlet import support
 from eventlet.green import BaseHTTPServer
 from eventlet.green import socket
-from eventlet.green import urllib
-from eventlet import greenio
-from eventlet import greenpool
-from eventlet import support
-from eventlet.support import six
+import six
+from six.moves import urllib
 
 
 DEFAULT_MAX_SIMULTANEOUS_REQUESTS = 1024
@@ -24,8 +24,15 @@ MINIMUM_CHUNK_SIZE = 4096
 # %(client_port)s is also available
 DEFAULT_LOG_FORMAT = ('%(client_ip)s - - [%(date_time)s] "%(request_line)s"'
                       ' %(status_code)s %(body_length)s %(wall_seconds).6f')
+RESPONSE_414 = b'''HTTP/1.0 414 Request URI Too Long\r\n\
+Connection: close\r\n\
+Content-Length: 0\r\n\r\n'''
 is_accepting = True
 
+STATE_IDLE = 'idle'
+STATE_REQUEST = 'request'
+STATE_CLOSE = 'close'
+
 __all__ = ['server', 'format_date_time']
 
 # Weekday and month names for HTTP date/time formatting; always English!
@@ -43,12 +50,25 @@ def format_date_time(timestamp):
     )
 
 
+def addr_to_host_port(addr):
+    host = 'unix'
+    port = ''
+    if isinstance(addr, tuple):
+        host = addr[0]
+        port = addr[1]
+    return (host, port)
+
+
 # Collections of error codes to compare against.  Not all attributes are set
 # on errno module on all platforms, so some are literals :(
 BAD_SOCK = set((errno.EBADF, 10053))
 BROKEN_SOCK = set((errno.EPIPE, errno.ECONNRESET))
 
 
+class ChunkReadError(ValueError):
+    pass
+
+
 # special flag return value for apps
 class _AlreadyHandled(object):
 
@@ -92,7 +112,19 @@ class Input(object):
         self.hundred_continue_headers = None
         self.is_hundred_continue_response_sent = False
 
+        # handle_one_response should give us a ref to the response state so we
+        # know whether we can still send the 100 Continue; until then, though,
+        # we're flying blind
+        self.headers_sent = None
+
     def send_hundred_continue_response(self):
+        if self.headers_sent:
+            # To late; application has already started sending data back
+            # to the client
+            # TODO: maybe log a warning if self.hundred_continue_headers
+            #       is not None?
+            return
+
         towrite = []
 
         # 100 Continue status line
@@ -108,19 +140,18 @@ class Input(object):
         towrite.append(b'\r\n')
 
         self.wfile.writelines(towrite)
+        self.wfile.flush()
 
         # Reinitialize chunk_length (expect more data)
         self.chunk_length = -1
 
     def _do_read(self, reader, length=None):
-        if self.wfile is not None and \
-                not self.is_hundred_continue_response_sent:
+        if self.wfile is not None and not self.is_hundred_continue_response_sent:
             # 100 Continue response
             self.send_hundred_continue_response()
             self.is_hundred_continue_response_sent = True
-        if length is None and self.content_length is not None:
-            length = self.content_length - self.position
-        if length and length > self.content_length - self.position:
+        if (self.content_length is not None) and (
+                length is None or length > self.content_length - self.position):
             length = self.content_length - self.position
         if not length:
             return b''
@@ -132,14 +163,13 @@ class Input(object):
         return read
 
     def _chunked_read(self, rfile, length=None, use_readline=False):
-        if self.wfile is not None and \
-                not self.is_hundred_continue_response_sent:
+        if self.wfile is not None and not self.is_hundred_continue_response_sent:
             # 100 Continue response
             self.send_hundred_continue_response()
             self.is_hundred_continue_response_sent = True
         try:
             if length == 0:
-                return ""
+                return b""
 
             if length and length < 0:
                 length = None
@@ -172,10 +202,13 @@ class Input(object):
                         length -= datalen
                         if length == 0:
                             break
-                    if use_readline and data[-1] == "\n":
+                    if use_readline and data[-1:] == b"\n":
                         break
                 else:
-                    self.chunk_length = int(rfile.readline().split(b";", 1)[0], 16)
+                    try:
+                        self.chunk_length = int(rfile.readline().split(b";", 1)[0], 16)
+                    except ValueError as err:
+                        raise ChunkReadError(err)
                     self.position = 0
                     if self.chunk_length == 0:
                         rfile.readline()
@@ -195,7 +228,17 @@ class Input(object):
             return self._do_read(self.rfile.readline, size)
 
     def readlines(self, hint=None):
-        return self._do_read(self.rfile.readlines, hint)
+        if self.chunked_input:
+            lines = []
+            for line in iter(self.readline, b''):
+                lines.append(line)
+                if hint and hint > 0:
+                    hint -= len(line)
+                    if hint <= 0:
+                        break
+            return lines
+        else:
+            return self._do_read(self.rfile.readlines, hint)
 
     def __iter__(self):
         return iter(self.read, b'')
@@ -215,6 +258,10 @@ class Input(object):
                 for key, value in headers]
         self.hundred_continue_headers = headers
 
+    def discard(self, buffer_size=16 << 10):
+        while self.read(buffer_size):
+            pass
+
 
 class HeaderLineTooLong(Exception):
     pass
@@ -224,6 +271,53 @@ class HeadersTooLarge(Exception):
     pass
 
 
+def get_logger(log, debug):
+    if callable(getattr(log, 'info', None)) \
+       and callable(getattr(log, 'debug', None)):
+        return log
+    else:
+        return LoggerFileWrapper(log or sys.stderr, debug)
+
+
+class LoggerNull(object):
+    def __init__(self):
+        pass
+
+    def error(self, msg, *args, **kwargs):
+        pass
+
+    def info(self, msg, *args, **kwargs):
+        pass
+
+    def debug(self, msg, *args, **kwargs):
+        pass
+
+    def write(self, msg, *args):
+        pass
+
+
+class LoggerFileWrapper(LoggerNull):
+    def __init__(self, log, debug):
+        self.log = log
+        self._debug = debug
+
+    def error(self, msg, *args, **kwargs):
+        self.write(msg, *args)
+
+    def info(self, msg, *args, **kwargs):
+        self.write(msg, *args)
+
+    def debug(self, msg, *args, **kwargs):
+        if self._debug:
+            self.write(msg, *args)
+
+    def write(self, msg, *args):
+        msg = msg + '\n'
+        if args:
+            msg = msg % args
+        self.log.write(msg)
+
+
 class FileObjectForHeaders(object):
 
     def __init__(self, fp):
@@ -248,9 +342,34 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
     minimum_chunk_size = MINIMUM_CHUNK_SIZE
     capitalize_response_headers = True
 
+    # https://github.com/eventlet/eventlet/issues/295
+    # Stdlib default is 0 (unbuffered), but then `wfile.writelines()` looses data
+    # so before going back to unbuffered, remove any usage of `writelines`.
+    wbufsize = 16 << 10
+
+    def __init__(self, conn_state, server):
+        self.request = conn_state[1]
+        self.client_address = conn_state[0]
+        self.conn_state = conn_state
+        self.server = server
+        self.setup()
+        try:
+            self.handle()
+        finally:
+            self.finish()
+
     def setup(self):
         # overriding SocketServer.setup to correctly handle SSL.Connection objects
         conn = self.connection = self.request
+
+        # TCP_QUICKACK is a better alternative to disabling Nagle's algorithm
+        # https://news.ycombinator.com/item?id=10607422
+        if getattr(socket, 'TCP_QUICKACK', None):
+            try:
+                conn.setsockopt(socket.IPPROTO_TCP, socket.TCP_QUICKACK, True)
+            except socket.error:
+                pass
+
         try:
             self.rfile = conn.makefile('rb', self.rbufsize)
             self.wfile = conn.makefile('wb', self.wbufsize)
@@ -261,33 +380,48 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
                 self.wfile = socket._fileobject(conn, "wb", self.wbufsize)
             else:
                 # it's a SSLObject, or a martian
-                raise NotImplementedError("wsgi.py doesn't support sockets "
-                                          "of type %s" % type(conn))
+                raise NotImplementedError(
+                    '''eventlet.wsgi doesn't support sockets of type {0}'''.format(type(conn)))
+
+    def handle(self):
+        self.close_connection = True
+
+        while True:
+            self.handle_one_request()
+            if self.conn_state[2] == STATE_CLOSE:
+                self.close_connection = 1
+            if self.close_connection:
+                break
+
+    def _read_request_line(self):
+        if self.rfile.closed:
+            self.close_connection = 1
+            return ''
+
+        try:
+            return self.rfile.readline(self.server.url_length_limit)
+        except greenio.SSL.ZeroReturnError:
+            pass
+        except socket.error as e:
+            last_errno = support.get_errno(e)
+            if last_errno in BROKEN_SOCK:
+                self.server.log.debug('({0}) connection reset by peer {1!r}'.format(
+                    self.server.pid,
+                    self.client_address))
+            elif last_errno not in BAD_SOCK:
+                raise
+        return ''
 
     def handle_one_request(self):
         if self.server.max_http_version:
             self.protocol_version = self.server.max_http_version
 
-        if self.rfile.closed:
+        self.raw_requestline = self._read_request_line()
+        if not self.raw_requestline:
             self.close_connection = 1
             return
-
-        try:
-            self.raw_requestline = self.rfile.readline(self.server.url_length_limit)
-            if len(self.raw_requestline) == self.server.url_length_limit:
-                self.wfile.write(
-                    b"HTTP/1.0 414 Request URI Too Long\r\n"
-                    b"Connection: close\r\nContent-length: 0\r\n\r\n")
-                self.close_connection = 1
-                return
-        except greenio.SSL.ZeroReturnError:
-            self.raw_requestline = ''
-        except socket.error as e:
-            if support.get_errno(e) not in BAD_SOCK:
-                raise
-            self.raw_requestline = ''
-
-        if not self.raw_requestline:
+        if len(self.raw_requestline) >= self.server.url_length_limit:
+            self.wfile.write(RESPONSE_414)
             self.close_connection = 1
             return
 
@@ -312,10 +446,12 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
             self.rfile = orig_rfile
 
         content_length = self.headers.get('content-length')
-        if content_length:
+        if content_length is not None:
             try:
-                int(content_length)
+                if int(content_length) < 0:
+                    raise ValueError
             except ValueError:
+                # Negative, or not an int at all
                 self.wfile.write(
                     b"HTTP/1.0 400 Bad Request\r\n"
                     b"Connection: close\r\nContent-length: 0\r\n\r\n")
@@ -339,6 +475,9 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
         start = time.time()
         headers_set = []
         headers_sent = []
+        # Push the headers-sent state into the Input so it won't send a
+        # 100 Continue response if we've already started a response.
+        self.environ['wsgi.input'].headers_sent = headers_sent
 
         wfile = self.wfile
         result = None
@@ -346,7 +485,7 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
         length = [0]
         status_code = [200]
 
-        def write(data, _writelines=wfile.writelines):
+        def write(data):
             towrite = []
             if not headers_set:
                 raise AssertionError("write() before start_response()")
@@ -395,24 +534,9 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
                 towrite.append(six.b("%x" % (len(data),)) + b"\r\n" + data + b"\r\n")
             else:
                 towrite.append(data)
-            try:
-                _writelines(towrite)
-                length[0] = length[0] + sum(map(len, towrite))
-            except UnicodeEncodeError:
-                self.server.log_message(
-                    "Encountered non-ascii unicode while attempting to write"
-                    "wsgi response: %r" %
-                    [x for x in towrite if isinstance(x, six.text_type)])
-                self.server.log_message(traceback.format_exc())
-                _writelines(
-                    ["HTTP/1.1 500 Internal Server Error\r\n",
-                     "Connection: close\r\n",
-                     "Content-type: text/plain\r\n",
-                     "Content-length: 98\r\n",
-                     "Date: %s\r\n" % format_date_time(time.time()),
-                     "\r\n",
-                     ("Internal Server Error: wsgi application passed "
-                      "a unicode object to the server instead of a string.")])
+            wfile.writelines(towrite)
+            wfile.flush()
+            length[0] = length[0] + sum(map(len, towrite))
 
         def start_response(status, response_headers, exc_info=None):
             status_code[0] = status.split()[0]
@@ -430,8 +554,15 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
             # Per HTTP RFC standard, header name is case-insensitive.
             # Please, fix your client to ignore header case if possible.
             if self.capitalize_response_headers:
+                if six.PY2:
+                    def cap(x):
+                        return x.capitalize()
+                else:
+                    def cap(x):
+                        return x.encode('latin1').capitalize().decode('latin1')
+
                 response_headers = [
-                    ('-'.join([x.capitalize() for x in key.split('-')]), value)
+                    ('-'.join([cap(x) for x in key.split('-')]), value)
                     for key, value in response_headers]
 
             headers_set[:] = [status, response_headers]
@@ -456,6 +587,11 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
                 minimum_write_chunk_size = int(self.environ.get(
                     'eventlet.minimum_write_chunk_size', self.minimum_chunk_size))
                 for data in result:
+                    if len(data) == 0:
+                        continue
+                    if isinstance(data, six.text_type):
+                        data = data.encode('ascii')
+
                     towrite.append(data)
                     towrite_size += len(data)
                     if towrite_size >= minimum_write_chunk_size:
@@ -471,8 +607,8 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
             except Exception:
                 self.close_connection = 1
                 tb = traceback.format_exc()
-                self.server.log_message(tb)
-                if not headers_set:
+                self.server.log.info(tb)
+                if not headers_sent:
                     err_body = six.b(tb) if self.server.debug else b''
                     start_response("500 Internal Server Error",
                                    [('Content-type', 'text/plain'),
@@ -481,25 +617,38 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
         finally:
             if hasattr(result, 'close'):
                 result.close()
-            if (self.environ['eventlet.input'].chunked_input or
-                    self.environ['eventlet.input'].position
-                    < (self.environ['eventlet.input'].content_length or 0)):
+            request_input = self.environ['eventlet.input']
+            if (request_input.chunked_input or
+                    request_input.position < (request_input.content_length or 0)):
                 # Read and discard body if there was no pending 100-continue
-                if not self.environ['eventlet.input'].wfile:
-                    # NOTE: MINIMUM_CHUNK_SIZE is used here for purpose different than chunking.
-                    # We use it only cause it's at hand and has reasonable value in terms of
-                    # emptying the buffer.
-                    while self.environ['eventlet.input'].read(MINIMUM_CHUNK_SIZE):
-                        pass
+                if not request_input.wfile and self.close_connection == 0:
+                    try:
+                        request_input.discard()
+                    except ChunkReadError as e:
+                        self.close_connection = 1
+                        self.server.log.error((
+                            'chunked encoding error while discarding request body.'
+                            + ' client={0} request="{1}" error="{2}"').format(
+                                self.get_client_address()[0], self.requestline, e,
+                        ))
+                    except IOError as e:
+                        self.close_connection = 1
+                        self.server.log.error((
+                            'I/O error while discarding request body.'
+                            + ' client={0} request="{1}" error="{2}"').format(
+                                self.get_client_address()[0], self.requestline, e,
+                        ))
             finish = time.time()
 
             for hook, args, kwargs in self.environ['eventlet.posthooks']:
                 hook(self.environ, *args, **kwargs)
 
             if self.server.log_output:
-                self.server.log_message(self.server.log_format % {
-                    'client_ip': self.get_client_ip(),
-                    'client_port': self.client_address[1],
+                client_host, client_port = self.get_client_address()
+
+                self.server.log.info(self.server.log_format % {
+                    'client_ip': client_host,
+                    'client_port': client_port,
                     'date_time': self.log_date_time_string(),
                     'request_line': self.requestline,
                     'status_code': status_code[0],
@@ -507,13 +656,14 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
                     'wall_seconds': finish - start,
                 })
 
-    def get_client_ip(self):
-        client_ip = self.client_address[0]
+    def get_client_address(self):
+        host, port = addr_to_host_port(self.client_address)
+
         if self.server.log_x_forwarded_for:
             forward = self.headers.get('X-Forwarded-For', '').replace(' ', '')
             if forward:
-                client_ip = "%s,%s" % (forward, client_ip)
-        return client_ip
+                host = forward + ',' + host
+        return (host, port)
 
     def get_environ(self):
         env = self.server.get_environ()
@@ -522,7 +672,10 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
 
         pq = self.path.split('?', 1)
         env['RAW_PATH_INFO'] = pq[0]
-        env['PATH_INFO'] = urllib.unquote(pq[0])
+        if six.PY2:
+            env['PATH_INFO'] = urllib.parse.unquote(pq[0])
+        else:
+            env['PATH_INFO'] = urllib.parse.unquote(pq[0], encoding='latin1')
         if len(pq) > 1:
             env['QUERY_STRING'] = pq[1]
 
@@ -539,11 +692,13 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
             env['CONTENT_LENGTH'] = length
         env['SERVER_PROTOCOL'] = 'HTTP/1.0'
 
-        host, port = self.request.getsockname()[:2]
-        env['SERVER_NAME'] = host
-        env['SERVER_PORT'] = str(port)
-        env['REMOTE_ADDR'] = self.client_address[0]
-        env['REMOTE_PORT'] = str(self.client_address[1])
+        sockname = self.request.getsockname()
+        server_addr = addr_to_host_port(sockname)
+        env['SERVER_NAME'] = server_addr[0]
+        env['SERVER_PORT'] = str(server_addr[1])
+        client_addr = addr_to_host_port(self.client_address)
+        env['REMOTE_ADDR'] = client_addr[0]
+        env['REMOTE_PORT'] = str(client_addr[1])
         env['GATEWAY_INTERFACE'] = 'CGI/1.1'
 
         try:
@@ -553,10 +708,11 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
         else:
             headers = [h.split(':', 1) for h in headers]
 
-        for k, v in headers:
+        env['headers_raw'] = headers_raw = tuple((k, v.strip(' \t\n\r')) for k, v in headers)
+        for k, v in headers_raw:
             k = k.replace('-', '_').upper()
-            v = v.strip()
-            if k in env:
+            if k in ('CONTENT_TYPE', 'CONTENT_LENGTH'):
+                # These do not get the HTTP_ prefix and were handled above
                 continue
             envk = 'HTTP_' + k
             if envk in env:
@@ -564,7 +720,7 @@ class HttpProtocol(BaseHTTPServer.BaseHTTPRequestHandler):
             else:
                 env[envk] = v
 
-        if env.get('HTTP_EXPECT') == '100-continue':
+        if env.get('HTTP_EXPECT', '').lower() == '100-continue':
             wfile = self.wfile
             wfile_line = b'HTTP/1.1 100 Continue\r\n'
         else:
@@ -615,10 +771,9 @@ class Server(BaseHTTPServer.HTTPServer):
         self.outstanding_requests = 0
         self.socket = socket
         self.address = address
-        if log:
-            self.log = log
-        else:
-            self.log = sys.stderr
+        self.log = LoggerNull()
+        if log_output:
+            self.log = get_logger(log, debug)
         self.app = app
         self.keepalive = keepalive
         self.environ = environ
@@ -658,26 +813,26 @@ class Server(BaseHTTPServer.HTTPServer):
             d.update(self.environ)
         return d
 
-    def process_request(self, sock_params):
+    def process_request(self, conn_state):
         # The actual request handling takes place in __init__, so we need to
         # set minimum_chunk_size before __init__ executes and we don't want to modify
         # class variable
-        sock, address = sock_params
         proto = new(self.protocol)
         if self.minimum_chunk_size is not None:
             proto.minimum_chunk_size = self.minimum_chunk_size
         proto.capitalize_response_headers = self.capitalize_response_headers
         try:
-            proto.__init__(sock, address, self)
+            proto.__init__(conn_state, self)
         except socket.timeout:
             # Expected exceptions are not exceptional
-            sock.close()
-            if self.debug:
-                # similar to logging "accepted" in server()
-                self.log_message('(%s) timed out %r' % (self.pid, address))
+            conn_state[1].close()
+            # similar to logging "accepted" in server()
+            self.log.debug('({0}) timed out {1!r}'.format(self.pid, conn_state[0]))
 
     def log_message(self, message):
-        self.log.write(message + '\n')
+        raise AttributeError('''\
+eventlet.wsgi.server.log_message was deprecated and deleted.
+Please use server.log.info instead.''')
 
 
 try:
@@ -696,6 +851,24 @@ except ImportError:
     ACCEPT_ERRNO = set((errno.EPIPE, errno.EBADF, errno.ECONNRESET))
 
 
+def socket_repr(sock):
+    scheme = 'http'
+    if hasattr(sock, 'do_handshake'):
+        scheme = 'https'
+
+    name = sock.getsockname()
+    if sock.family == socket.AF_INET:
+        hier_part = '//{0}:{1}'.format(*name)
+    elif sock.family == socket.AF_INET6:
+        hier_part = '//[{0}]:{1}'.format(*name[:2])
+    elif sock.family == socket.AF_UNIX:
+        hier_part = name
+    else:
+        hier_part = repr(name)
+
+    return scheme + ':' + hier_part
+
+
 def server(sock, site,
            log=None,
            environ=None,
@@ -733,10 +906,12 @@ def server(sock, site,
 
     :param sock: Server socket, must be already bound to a port and listening.
     :param site: WSGI application function.
-    :param log: File-like object that logs should be written to.
+    :param log: logging.Logger instance or file-like object that logs should be written to.
+                If a Logger instance is supplied, messages are sent to the INFO log level.
                 If not specified, sys.stderr is used.
     :param environ: Additional parameters that go into the environ dictionary of every request.
     :param max_size: Maximum number of client connections opened at any time by this server.
+                Default is 1024.
     :param max_http_version: Set to "HTTP/1.0" to make the server pretend it only supports HTTP 1.0.
                 This can help with applications or clients that don't behave properly using HTTP 1.1.
     :param protocol: Protocol class.  Deprecated.
@@ -766,68 +941,72 @@ def server(sock, site,
     :param capitalize_response_headers: Normalize response headers' names to Foo-Bar.
                 Default is True.
     """
-    serv = Server(sock, sock.getsockname(),
-                  site, log,
-                  environ=environ,
-                  max_http_version=max_http_version,
-                  protocol=protocol,
-                  minimum_chunk_size=minimum_chunk_size,
-                  log_x_forwarded_for=log_x_forwarded_for,
-                  keepalive=keepalive,
-                  log_output=log_output,
-                  log_format=log_format,
-                  url_length_limit=url_length_limit,
-                  debug=debug,
-                  socket_timeout=socket_timeout,
-                  capitalize_response_headers=capitalize_response_headers,
-                  )
+    serv = Server(
+        sock, sock.getsockname(),
+        site, log,
+        environ=environ,
+        max_http_version=max_http_version,
+        protocol=protocol,
+        minimum_chunk_size=minimum_chunk_size,
+        log_x_forwarded_for=log_x_forwarded_for,
+        keepalive=keepalive,
+        log_output=log_output,
+        log_format=log_format,
+        url_length_limit=url_length_limit,
+        debug=debug,
+        socket_timeout=socket_timeout,
+        capitalize_response_headers=capitalize_response_headers,
+    )
     if server_event is not None:
+        warnings.warn(
+            'eventlet.wsgi.Server() server_event kwarg is deprecated and will be removed soon',
+            DeprecationWarning, stacklevel=2)
         server_event.send(serv)
     if max_size is None:
         max_size = DEFAULT_MAX_SIMULTANEOUS_REQUESTS
     if custom_pool is not None:
         pool = custom_pool
     else:
-        pool = greenpool.GreenPool(max_size)
-    try:
-        host, port = sock.getsockname()[:2]
-        port = ':%s' % (port, )
-        if hasattr(sock, 'do_handshake'):
-            scheme = 'https'
-            if port == ':443':
-                port = ''
-        else:
-            scheme = 'http'
-            if port == ':80':
-                port = ''
+        pool = eventlet.GreenPool(max_size)
 
-        serv.log.write("(%s) wsgi starting up on %s://%s%s/\n" % (
-            serv.pid, scheme, host, port))
+    if not (hasattr(pool, 'spawn') and hasattr(pool, 'waitall')):
+        raise AttributeError('''\
+eventlet.wsgi.Server pool must provide methods: `spawn`, `waitall`.
+If unsure, use eventlet.GreenPool.''')
+
+    # [addr, socket, state]
+    connections = {}
+
+    def _clean_connection(_, conn):
+        connections.pop(conn[0], None)
+        conn[2] = STATE_CLOSE
+        greenio.shutdown_safe(conn[1])
+        conn[1].close()
+
+    try:
+        serv.log.info('({0}) wsgi starting up on {1}'.format(serv.pid, socket_repr(sock)))
         while is_accepting:
             try:
-                client_socket = sock.accept()
-                client_socket[0].settimeout(serv.socket_timeout)
-                if debug:
-                    serv.log.write("(%s) accepted %r\n" % (
-                        serv.pid, client_socket[1]))
-                try:
-                    pool.spawn_n(serv.process_request, client_socket)
-                except AttributeError:
-                    warnings.warn("wsgi's pool should be an instance of "
-                                  "eventlet.greenpool.GreenPool, is %s. Please convert your"
-                                  " call site to use GreenPool instead" % type(pool),
-                                  DeprecationWarning, stacklevel=2)
-                    pool.execute_async(serv.process_request, client_socket)
+                client_socket, client_addr = sock.accept()
+                client_socket.settimeout(serv.socket_timeout)
+                serv.log.debug('({0}) accepted {1!r}'.format(serv.pid, client_addr))
+                connections[client_addr] = connection = [client_addr, client_socket, STATE_IDLE]
+                (pool.spawn(serv.process_request, connection)
+                    .link(_clean_connection, connection))
             except ACCEPT_EXCEPTIONS as e:
                 if support.get_errno(e) not in ACCEPT_ERRNO:
                     raise
             except (KeyboardInterrupt, SystemExit):
-                serv.log.write("wsgi exiting\n")
+                serv.log.info('wsgi exiting')
                 break
     finally:
+        for cs in six.itervalues(connections):
+            prev_state = cs[2]
+            cs[2] = STATE_CLOSE
+            if prev_state == STATE_IDLE:
+                greenio.shutdown_safe(cs[1])
         pool.waitall()
-        serv.log.write("(%s) wsgi exited, is_accepting=%s\n" % (
-            serv.pid, is_accepting))
+        serv.log.info('({0}) wsgi exited, is_accepting={1}'.format(serv.pid, is_accepting))
         try:
             # NOTE: It's not clear whether we want this to leave the
             # socket open or close it.  Use cases like Spawning want
