@@ -9,27 +9,30 @@ Connect to it with:
 You terminate your connection by terminating telnet (typically Ctrl-]
 and then 'quit')
 """
+from __future__ import print_function
 
 import eventlet
 
+
 def handle(fd):
-    print "client connected"
+    print("client connected")
     while True:
         # pass through every non-eof line
         x = fd.readline()
-        if not x: break
+        if not x:
+            break
         fd.write(x)
         fd.flush()
-        print "echoed", x,
-    print "client disconnected"
+        print("echoed", x, end=' ')
+    print("client disconnected")
 
-print "server socket listening on port 6000"
+print("server socket listening on port 6000")
 server = eventlet.listen(('0.0.0.0', 6000))
 pool = eventlet.GreenPool()
 while True:
     try:
         new_sock, address = server.accept()
-        print "accepted", address
+        print("accepted", address)
         pool.spawn_n(handle, new_sock.makefile('rw'))
     except (SystemExit, KeyboardInterrupt):
         break
